#import <Foundation/Foundation.h>

typedef BOOL(^BLKTypePred1)(id);

typedef id(^BLKTypeFn1)(id);

typedef id(^BLKTypeFn1Idx)(id, NSUInteger);

typedef id(^BLKTypeFn2)(id, id);

typedef id(^BLKTypeFn2Idx)(id, id, NSUInteger);

typedef id(^BLKTypeFn3)(id, id, id);

typedef id(^BLKTypeFn3Idx)(id, id, id, NSUInteger);

// return object to stop an enumeration. Doesn't work on filter (it always returns BOOL)
extern const id BLKStopProcessing;

@interface BLKFunction : NSObject

/**
* With the exception of objects without a mutable counterpart (e.g. NSNumber),
* these methods will return mutable versions of collections and objects.
* If this is not what you want, make a immutable copy!
*
* Reasoning - methods such as conjoin might be called within a fold. Other methods might
* be chained together. We are not using "persistent" data structures, so you must
* decide if and/or when its time for immutability.
*/
#pragma mark - Return mutable collections or objects

+ (NSMutableArray *)takeFromArray:(NSArray *)array numberOfItems:(NSUInteger)nrToTake;

+ (NSMutableArray *)dropFromArray:(NSArray *)array numberOfItems:(NSUInteger)nrToDrop;

+ (NSMutableArray *)filterArray:(NSArray *)array withBlock:(BLKTypePred1)block;

+ (NSMutableArray *)mapArray:(NSArray *)array withBlock:(BLKTypeFn1)block;

+ (id)reduceArray:(NSArray *)array withBlock:(BLKTypeFn2)block;

+ (id)foldArray:(NSArray *)array toAccumulator:(id)accumulator withBlock:(BLKTypeFn2)block;

#pragma mark - Conjoin is Experimental, functionality may change
/**
* Conjoin an object or collection with another object or collection.
* Joins Arrays, Sets, Numbers, Dictionaries and Objects.
* If cage1 is nil, it returns nil.
* Cage1 determines the return type.
* It pours cage2 into a "mutable" cage1 (if cage1 is immutable, a mutable copy is made)
*/
+ (id)conjoin:(id)cage1 and:(id)cage2;

+ (id)conjoin:(id)cage1 and:(id)cage2 and:(id)cage3;

+ (id)conjoin:(id)cage1 and:(id)cage2 interpose:(id)theInterpose;

+ (id)conjoin:(id)cage1 and:(id)cage2 interpose:(id)theInterpose end:(id)theEnd;

+ (id)conjoin:(id)cage1 and:(id)cage2 interleave:(NSArray *)theInterleave;

#pragma mark - The following methods may be categorized (use them directly, for now!)

+ (id)foldDictionary:(NSDictionary *)dict toAccumulator:(id)accumulator withBlock:(BLKTypeFn3)block;

#pragma mark - The following methods wont be categorized (use them directly!)

+ (NSMutableArray *)shuffleArray:(NSArray *)array;

+ (NSMutableArray *)reverseArray:(NSArray *)array;

+ (NSMutableArray *)mapArray:(NSArray *)array withIndexedBlock:(BLKTypeFn1Idx)block;

+ (id)foldArray:(NSArray *)array toAccumulator:(id)accumulator withIndexedBlock:(BLKTypeFn2Idx)block;

+ (id)foldDictionary:(NSDictionary *)dict
       toAccumulator:(id)accumulator withIndexedBlock:(BLKTypeFn3Idx)block;

+ (id)foldArray:(NSArray *)array usingOptions:(NSEnumerationOptions)options
  toAccumulator:(id)accumulator withIndexedBlock:(BLKTypeFn2Idx)block;

+ (id)foldDictionary:(NSDictionary *)dict usingOptions:(NSEnumerationOptions)options
       toAccumulator:(id)accumulator withIndexedBlock:(BLKTypeFn3Idx)block;

@end