#import <XCTest/XCTest.h>
#import "BLKCatFunction.h"

@interface BLKTest : XCTestCase
@end

@implementation BLKTest

/**
* The main tests are in the ExampleBLKFunctional project, for the moment!
* NOTE: It needs to be loaded separately into XCode / AppCode, by opening ExampleBLKFunctional.xcworkspace
*/
- (void)testCategorized {
    NSArray *numberArray = @[@2, @9, @66];
    NSArray *stringArray = @[@"one", @"two", @"three"];

    NSArray *theTest;

    // take, drop
    theTest = [numberArray blk_take:2];
    XCTAssertEqualObjects(theTest, (@[@2, @9]), @"take 2");

    theTest = [numberArray blk_drop:2];
    XCTAssertEqualObjects(theTest, (@[@66]), @"drop 2");

    // Filter
    theTest = [numberArray blk_filter:BLKPred1([p1 intValue] < 10)];
    XCTAssertEqualObjects(theTest, (@[@2, @9]), @"filter out numbers > 10");

    theTest = [@[@"hello", @"world"] blk_filter:BLKPredS([p1 isEqualToString:@"world"])];
    XCTAssertEqualObjects(theTest, (@[@"world"]), @"filter out non world strings");

    theTest = [@[@"hello", @"world"] blk_filter:BLKPredN(p1.intValue > 66) ];
    XCTAssertEqualObjects(theTest, (@[]), @"filter out everything");

    // Map
    theTest = [stringArray blk_map:BLKFn1([p1 uppercaseString]) ];
    XCTAssertEqualObjects(theTest, (@[@"ONE", @"TWO", @"THREE"]), @"uppercase using macro");

    theTest = [stringArray blk_map:BLKFnS([p1 uppercaseString]) ];
    XCTAssertEqualObjects(theTest, (@[@"ONE", @"TWO", @"THREE"]), @"uppercase using macro");

    theTest = [numberArray blk_map:BLKFnN(@(p1.doubleValue * 2)) ];
    XCTAssertEqualObjects(theTest, (@[@4, @18, @132]), @"multiply 2 using macro");

    // Reduce
    theTest = [numberArray blk_reduce:BLKFn2(@([p1 doubleValue] + [p2 doubleValue])) ];
    XCTAssertEqualObjects(theTest, @77, @"p1 + p2");

    theTest = [numberArray blk_reduce:BLKFnNN(@(p1.doubleValue + p2.doubleValue)) ];
    XCTAssertEqualObjects(theTest, @77, @"p1 + p2");

    theTest = [stringArray blk_reduce:BLKFnSS([p1 stringByAppendingString:p2]) ];
    XCTAssertEqualObjects(theTest, @"onetwothree", @"p1 + p2");

    // Fold
    theTest = [numberArray blk_fold:@2 with:BLKFn2(@([p1 doubleValue] + [p2 doubleValue])) ];
    XCTAssertEqualObjects(theTest, @79, @"p1 + p2");

    theTest = [numberArray blk_fold:@[] with:BLKFnAN(@[p2]) ];
    XCTAssertEqualObjects(theTest, @[@66], @"last element");

    theTest = [stringArray blk_fold:@[@"start"] with:BLKFnAS([p1 arrayByAddingObject:p2]) ];
    XCTAssertEqualObjects(theTest, (@[@"start", @"one", @"two", @"three"]), @"conjoin elements");

    // Conjoin
    theTest = [@[@1, @2]  blk_conjoin:@[@5, @6]];
    XCTAssertEqualObjects(theTest, (@[@1, @2, @5, @6]), @"conjoin array with array");

}

- (void)testNonCategorized {
    NSArray *numberArray = @[@2, @9, @66];

    NSArray *theTest;

    theTest = [BLKFunction shuffleArray:numberArray];
    XCTAssertTrue([theTest count] == numberArray.count, "non deterministic function");

    theTest = [BLKFunction reverseArray:numberArray];
    XCTAssertEqualObjects(theTest, (@[@66, @9, @2]), @"reverse");

    theTest = [BLKFunction mapArray:numberArray withIndexedBlock:^id(NSNumber *p1, NSUInteger idx) {
        return @(p1.doubleValue + idx);
    }];
    XCTAssertEqualObjects(theTest, (@[@2, @10, @68]), @"array element + index");

    theTest = [BLKFunction foldArray:numberArray toAccumulator:@5
                    withIndexedBlock:^(NSNumber *p1, NSNumber *p2, NSUInteger idx) {
                        return [BLKFunction conjoin:p1 and:p2];
    }];
    XCTAssertEqualObjects(theTest, @82, @"p1 + p2");
}

- (void)testDictionary {
    id theTest = [BLKFunction foldDictionary:@{@3 : @4, @5 : @6} usingOptions:0 toAccumulator:@""
                                  withIndexedBlock:^(id acc, NSString *obj1, NSString *obj2, NSUInteger idx) {
                                      return [BLKFunction conjoin:acc and:@[obj1, @":", obj2, @", "]];
                                  }];
    XCTAssertEqualObjects(theTest, @"3:4, 5:6, ", @"Add dictionary keys and values to accumulator");

    theTest = [BLKFunction foldDictionary:@{@3 : @4, @5 : @6} toAccumulator:@""
                            withIndexedBlock:^(id acc, NSString *obj1, NSString *obj2, NSUInteger idx) {
                                return [BLKFunction conjoin:acc and:@[obj1, @":", obj2, @", "]];
                            }];
    XCTAssertEqualObjects(theTest, @"3:4, 5:6, ", @"Add dictionary keys and values to accumulator");
}

@end
