#import <XCTest/XCTest.h>
#import <BLKFunctional/BLKCatFunction.h>

/**
* Tests for long versions of the Function methods, just to confirm they are correctly defined. Most of
 * the testing will done against the shorter method signatures.
* There are no macros to test.
*/
@interface BlkTestConjoin : XCTestCase

@end

@implementation BlkTestConjoin

- (void)testConjoin {

    id theTest;
    id thearr;

    // test direct
    thearr = @[@1, @2];
    theTest = [thearr blk_conjoin:@[@5, @6]];
    XCTAssertEqualObjects(theTest, (@[@1, @2, @5, @6]), @"conjoin set with array");

    // string
    theTest = [BLKFunction conjoin:@"one" and:@[@"two", @"three"]];
    XCTAssertEqualObjects(theTest, @"onetwothree", @"conjoin strings");

    // numbers
    theTest = [BLKFunction conjoin:@5 and:@6];
    XCTAssertEqualObjects(theTest, @11, @"conjoin numbers");

    theTest = [BLKFunction conjoin:@5 and:@[@6, @7]];
    XCTAssertEqualObjects(theTest, @18, @"conjoin to an array of numbers");

    // dictionaries
    theTest = [BLKFunction conjoin:@{@1 : @2} and:@{@3 : @4}];
    XCTAssertEqualObjects(theTest, (@{@1 : @2, @3 : @4}), @"conjoin two dictionaries");

    theTest = [BLKFunction conjoin:@[] and:@{@3 : @4, @5 : @6}];
    XCTAssertEqualObjects(theTest, (@[@[@3, @4], @[@5, @6]]), @"conjoin array with dictionary");

    theTest = [BLKFunction conjoin:@{} and:@[@[@3, @4], @[@5, @6]]];
    XCTAssertEqualObjects(theTest, (@{@3 : @4, @5 : @6}), @"conjoin dictionary with array");

    theTest = [BLKFunction conjoin:@{} and:@[@[@3, @4], @[@5, @6]]];
    XCTAssertEqualObjects(theTest, (@{@3 : @4, @5 : @6}), @"conjoin dictionary with array");

    theTest = [BLKFunction conjoin:@{} and:@[@3, @4, @5, @6]];
    XCTAssertEqualObjects(theTest, (@{@3 : @4, @5 : @6}), @"conjoin dictionary with flat array");

    // sets
    thearr = [[NSSet alloc] initWithObjects:@1, @2, @3, nil];
    id expected = [[NSSet alloc] initWithObjects:@1, @2, @3, @4, nil];
    theTest = [BLKFunction conjoin:thearr and:@4];
    XCTAssertEqualObjects(theTest, expected, @"conjoin set with object");

    theTest = [BLKFunction conjoin:@4 and:thearr];
    XCTAssertEqualObjects(theTest, @10, @"conjoin object with set");

}

@end
